/*
 * packetmodeller/message.h
 * $Rev: 846 $
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __MESSAGE_H
#define __MESSAGE_H

/**
 * @file message.h
 * @brief Message definitions for WlanModel <-> PacketModeller communication
 */

/* INCLUDES *******************************************************************/

#include <netinet/in.h>                 // struct sockaddr_in
#include <linux/if.h>                   // IFNAMSIZ
#include <net/ethernet.h>               // struct ether_header, ETH_ALEN
#include <limits.h>                     // for HOST_NAME_MAX
#include <stdint.h>                     // uint8_t, uint16_t, int16_t, ...


/* MESSAGE TYPES DEFINITIONS **************************************************/

/** Maximum size of an original wireless Ethernet frame */
#define MAX_FRAME_SIZE          (1500 + sizeof(struct ether_header))
// maximal size of an Ethernet frame (MTU 1500);

/** Wireless frame data structure */
typedef char raw_packet [MAX_FRAME_SIZE];

/**
 *  Message header
 */
struct msg_header {
    uint8_t type;                   /**< Message type */
    char id[2 * ETH_ALEN + 1];      /**< Sender identification */
};

/**
 *  Data message (encapsulated wireless packet)
 */
struct data_msg {
    struct msg_header header;       /**< Message header */
    uint16_t vif_index;             /**< Index of interface which captured this frame */
    uint16_t pkt_length;            /**< Length of Ethernet frame */
    raw_packet wlan_pkt;            /**< Original Ethernet frame */
} __attribute__((__packed__));      // pack structs to provide padding problems

/**
 *  Virtual device parameter format for config_msg
 */
typedef union vif_param {
    int16_t     txpower;            /**< Wireless radio transmit power [mW] */
    int16_t     retry;              /**< Retry limit */
    int16_t     rts;                /**< Ready-to-Send threshold [Bytes] */
    int16_t     sensitivity;        /**< Radio sensitivity [dB] */ //(negative)
    double      freq;               /**< Radio frequency/Channel [Hz] */
}
vif_param;

/**
 *  Configuration information
 */
struct s_vif_config {
    vif_param   value;              /**< Wireless parameter value */
    uint16_t    vif_index;          /**< Virtual wireless device index */
    uint16_t    type;               /**< Ioctl-type */
} __attribute__((__packed__));      // pack structs to provide padding problems
typedef struct s_vif_config vif_config;

/**
 *  Configuration message
 */
struct config_msg {
    struct msg_header   header;     /**< Message header */
    vif_config          vif_data;   /**< Configuration data */
};

/**
 *  Virtual device parameter format for registration_msg
 */
struct s_vif_registration {
    char            vif_name[IFNAMSIZ + 1]; /**< Virtual wireless device name */
    unsigned char   vif_mac[ETH_ALEN];  /**< Virtual wireless device MAC address */
    int16_t         vif_index;          /**< Virtual wireless device index */
    int16_t         txpower;            /**< Wireless radio transmit power [mW] */
    int16_t         retry;              /**< Retry limit */
    int16_t         rts;                /**< Ready-to-Send threshold [Bytes] */
    int16_t         sensitivity;        /**< Radio sensitivity [dB] */ //(negative)
    int64_t         freq;               /**< Radio frequency/Channel [Hz] */
} __attribute__((__packed__));          // pack structs to provide padding problems
typedef struct s_vif_registration vif_registration;

/**
 *  Registration information
 */
struct registration_info {
    char                name[HOST_NAME_MAX];    /**< Client host name */
    struct              sockaddr_storage addr;  /**< Client IP address (IPv4 or IPv6) */
    uint16_t            msg_id;                 /**< Message identification */
    uint16_t            num_vif;                /**< Number of virtual devices */
    vif_registration    vif_data[2];            /**> Information about wireless devices */
} __attribute__((__packed__));          // pack structs to provide padding problems

/**
 *  Registration message
 */
struct registration_msg {
    struct msg_header header;           /**< Message header */
    struct registration_info info;      /**< Registration information */
};

/**
 *  Plain message (only has id; for ack and de-register)
 */
struct plain_msg {
    struct msg_header header;           /**< Message header */
    uint8_t msg_id;                     /**< Message identification */
};


/* MESSAGE PARAMETER DEFINITIONS **********************************************/

#define DATA_MSG                0
#define DATA_PREAMBLE_SIZE      (sizeof(struct msg_header) + 2 * sizeof(uint16_t))
#define DATA_MSG_SIZE           sizeof(struct data_msg)

#define CONFIG_MSG              1
#define CONFIG_MSG_SIZE         sizeof(struct config_msg)

#define REGISTRATION_MSG        2
#define REGISTRATION_MSG_SIZE   sizeof(struct registration_msg)

#define PLAIN_MSG_SIZE          sizeof(struct plain_msg)

#define ACK_MSG                 3
#define ACK_MSG_SIZE            PLAIN_MSG_SIZE

#define DEREGISTRATION_MSG      4
#define DEREGISTRATION_MSG_SIZE PLAIN_MSG_SIZE


/** Maximal size of a PacketModeller <-> WlanModel message */
#define MAX_MSG_SIZE            max(WLAN_MSG_SIZE, REGISTRATION_MSG_SIZE)


/* PUBLIC MESSAGE FUNCTIONS ***************************************************/

int
is_data_msg(struct msg_header *);

int
is_registration_msg(struct msg_header *);

int
is_deregistration_msg(struct msg_header *);

int
is_ack_msg(struct msg_header *);

int
is_config_msg(struct msg_header *);

void
debug_msg(const char *);

#endif
